package edu.ntnu.stud.chaosgame.controller.game;

import javafx.scene.shape.Rectangle;

public class QuadtreeNode {
  private QuadtreeNode parent;
  private QuadtreeNode[] children;
  private boolean isLeaf;
  private int levelOfDetail;
  private Rectangle bounds;

  public QuadtreeNode(Rectangle bounds, int levelOfDetail) {
    this.bounds = bounds;
    this.levelOfDetail = levelOfDetail;
    this.isLeaf = true;
    this.children = new QuadtreeNode[4];
  }

  public void split() {
    if (!isLeaf) {
      return;
    }

    double halfWidth = bounds.getWidth() / 2;
    double halfHeight = bounds.getHeight() / 2;

    children[0] = new QuadtreeNode(new Rectangle(bounds.getX(), bounds.getY(), halfWidth, halfHeight), levelOfDetail + 1);
    children[1] = new QuadtreeNode(new Rectangle(bounds.getX() + halfWidth, bounds.getY(), halfWidth, halfHeight), levelOfDetail + 1);
    children[2] = new QuadtreeNode(new Rectangle(bounds.getX(), bounds.getY() + halfHeight, halfWidth, halfHeight), levelOfDetail + 1);
    children[3] = new QuadtreeNode(new Rectangle(bounds.getX() + halfWidth, bounds.getY() + halfHeight, halfWidth, halfHeight), levelOfDetail + 1);

    isLeaf = false;
  }

  public void merge() {
    if (isLeaf) {
      return;
    }

    for (int i = 0; i < 4; i++) {
      children[i] = null;
    }

    isLeaf = true;
  }

  public Rectangle getBounds() {
    return this.bounds;
  }

  public QuadtreeNode[] getChildren() {
    return this.children;
  }

  public Boolean isLeaf() {
    return this.isLeaf;
  }

  // getters and setters omitted for brevity
}
